   title "Incremental 2 Pulse Convertor - v.1.0"

   list p=12c508a, F=INHX8M
   __config 0Ah           ; MCLR off, CP off, WDT off, OSC int RC
; ---------------------------------------------------------------------
;
; This software is free for private usage. It was created for HAM radio
; community members. Commercial exploatation is allowed only with
; permission of author.
;
; (C) Petr Bittnar, OK1MPE
;     ok1mpe@qsl.net, www.qsl.net/ok1mpe
;
; ---------------------------------------------------------------------
;
;**********************************************************************
;*  RC int., Tcpu=1us, 12C508A, MPASM 2.15                   08.04.01 *
;**********************************************************************
;
;      Vdd   -| U |- Vss
; cw pulse   -|   |- channel A
; ccw pulse  -|   |- channel B
; pulse 25ms -|   |- not use

c          equ 0          ;def. status
dc         equ 1
z          equ 2
rp0        equ 5
ina        equ 0          ;def. gpio
inb        equ 1
pulse25    equ 3
outccw     equ 4
outcw      equ 5
cw         equ 0          ;def. flag
ccw        equ 1

;def. RAM (registers):
indir      equ 0
rtcc       equ 1
pc         equ 2
status     equ 3
fsr        equ 4
osccal     equ 5
gpio       equ 6

flag       equ 7
old        equ 8
new        equ 9
cha        equ 0A
chb        equ 0B

   org     0              ;reset vector
   movwf   osccal         ;calibration value for RC osc.
   goto    start

;**********************************************************************
;* init settings - ports, prescaler,...                               *
;**********************************************************************
;ports: 0-output, 1-input
start
   movlw   b'001111'      ;ina, inb, pulse25 input
   tris    gpio           ;outcw, out ccw output
   clrf    gpio

   movlw   b'00000100'    ;ps 1:32, pull-up
   option

   clrf    flag           ;clear registers
   clrf    old            ;
   clrf    new            ;

;----------------------------------------------------------------------
detent
   clrf    cha            ;clear registers
   clrf    chb            ;
   movfw   gpio           ;read channels
   andlw   b'00000011'    ;
   movwf   new            ;
   btfsc   new,ina        ;channel A
   bsf     cha,0
   btfsc   new,inb        ;channel B
   bsf     chb,0

   movfw   cha            ;
   xorwf   chb,0          ;compare channels A, B
   btfss   status,z       ;same?
   goto    change         ;no

   btfsc   flag,cw        ;
   goto    valid          ;
   btfsc   flag,ccw       ;
   goto    valid          ;

   movfw   new            ;new > old
   movwf   old            ;
   goto    detent

;----------------------------------------------------------------------
valid
   rrf     new,0
   andlw   b'00000001'    ;
   xorwf   old,0          ;compare log. levels - channel B
   btfss   status,z       ;different (valid detent position)?
   call    pulse          ;yes gen. pulse
   goto    detent         ;

;----------------------------------------------------------------------
change
   btfsc   flag,cw        ;continue only if it's first change
   goto    detent         ;(flags cw=0 and ccw=0)
   btfsc   flag,ccw       ;
   goto    detent         ;

   movfw   old            ;channel A
   andlw   b'00000001'    ;
   xorwf   cha,0          ;compare two last reading
   btfss   status,z       ;same?
   bsf     flag,cw        ;no change on the channel A (set cw=1)

   rrf     old,0          ;channel B
   andlw   b'00000001'    ;
   movwf   old
   xorwf   chb,0          ;compare two last reading
   btfss   status,z       ;same?
   bsf     flag,ccw       ;no change on the channel B (set ccw=1)

   call    delay          ;wait a moment
   goto    detent         ;

;----------------------------------------------------------------------
;generation pulse
;
pulse
   btfsc   flag,cw        ;cw rotation?
   bsf     gpio,outcw     ;yes, pulse on the cw pin
   btfsc   flag,ccw       ;ccw rotation?
   bsf     gpio,outccw    ;yes, pulse on the ccw pin

   btfsc   gpio,pulse25   ;pulse delay 25ms?
   goto    $+4            ;no skip
   call    delay          ;pulse 25ms (15+10)
   call    delay
   call    delay
   call    delay          ;pulse 10ms
   call    delay

   clrf    gpio           ;set init values
   clrf    flag           ;(detent position)
   retlw   0

;----------------------------------------------------------------------
;delay subrutine - 5ms
;
delay
   movlw   .100           ;(256-100)x32us
   movwf   rtcc           ;rtcc=5ms

   movfw   rtcc           ;read rtcc
   btfss   status,z       ;overflow rtcc?
   goto    $-2            ;ne repeat
   retlw   0              ;

   end
